function initializeCoreMod() {
	return {
        'player_init_hook': {
            'target': {
                'type': 'CLASS',
                'name': 'net.minecraft.block.Block'
            },
            'transformer': function(classNode) {
                log("Patching Block...");
                patch({
                    obfName: "func_176225_a",
                    name: "shouldSideBeRendered",
                    desc: "(Lnet/minecraft/block/BlockState;Lnet/minecraft/world/IBlockReader;Lnet/minecraft/util/math/BlockPos;Lnet/minecraft/util/Direction;)Z",
                    patch: patch_Block_shouldSideBeRendered
                }, classNode);
                return classNode;
            }
        }
    };
}

function findMethod(methods, entry) {
    var length = methods.length;
    for(var i = 0; i < length; i++) {
        var method = methods[i];
        if((method.name.equals(entry.obfName) || method.name.equals(entry.name)) && method.desc.equals(entry.desc)) {
            return method;
        }
    }
    return null;
}

function patch(entry, classNode) {
    var method = findMethod(classNode.methods, entry);
    var name = classNode.name.replace("/", ".") + "#" + entry.name + entry.desc;
    if(method !== null) {
        log("Starting to patch: " + name);
        if(entry.patch(method)) {
            log("Successfully patched: " + name);
        } else {
            log("Failed to patch: " + name);
        }
    } else {
        log("Failed to find method: " + name);
    }
}

var ASMAPI = Java.type('net.minecraftforge.coremod.api.ASMAPI');
var Opcodes = Java.type('org.objectweb.asm.Opcodes');
var MethodInsnNode = Java.type('org.objectweb.asm.tree.MethodInsnNode');
var InsnNode = Java.type('org.objectweb.asm.tree.InsnNode');
var VarInsnNode = Java.type('org.objectweb.asm.tree.VarInsnNode');
var IntInsnNode = Java.type('org.objectweb.asm.tree.IntInsnNode');
var LabelNode = Java.type('org.objectweb.asm.tree.LabelNode');
var FieldInsnNode = Java.type('org.objectweb.asm.tree.FieldInsnNode');
var TypeInsnNode = Java.type('org.objectweb.asm.tree.TypeInsnNode');
var JumpInsnNode = Java.type('org.objectweb.asm.tree.JumpInsnNode');
var FrameNode = Java.type('org.objectweb.asm.tree.FrameNode');

function patch_Block_shouldSideBeRendered(method) {
    var firstInsn = method.instructions.get(0);
    method.instructions.insertBefore(firstInsn, new VarInsnNode(Opcodes.ALOAD, 0));
    method.instructions.insertBefore(firstInsn, new VarInsnNode(Opcodes.ALOAD, 3));
    method.instructions.insertBefore(firstInsn, new MethodInsnNode(Opcodes.INVOKESTATIC, "com/mrcrayfish/slopes/Slopes", "canRenderSide", "(Lnet/minecraft/block/BlockState;Lnet/minecraft/util/Direction;)Z", false));
    var label = new LabelNode();
    method.instructions.insertBefore(firstInsn, new JumpInsnNode(Opcodes.IFEQ, label));
    method.instructions.insertBefore(firstInsn, new InsnNode(Opcodes.ICONST_1));
    method.instructions.insertBefore(firstInsn, new InsnNode(Opcodes.IRETURN));
    method.instructions.insertBefore(firstInsn, label);
    method.instructions.insertBefore(firstInsn, new FrameNode(Opcodes.F_SAME, 0, null, 0, null));
    return true;
}

function log(s) {
    print("[slopes-transformer.js] " + s);
}